package ucl.entities;

import java.util.HashMap;
import java.util.Map;

import repast.simphony.engine.schedule.ScheduledMethod;
import ucl.model.AgentSettlementModel;
import ucl.model.Model;

/**
 * The Agent class (could be a household or person)
 * 
 * @author 
 *
 */
public class Agent {

	/**The original settlement id number the agent begins at**/
	public int originalSettlement;
	
	/**The agent's id number**/
	private int agentNumber;
	
	/**The current settlement the agent is located at*/
	public Settlement settlement;
	
	/**The model used for settlement interaction and decision to move or not*/
	private Model model;
	
	/**The flow an agent can achieve by interacting with settlements*/
	public double flow;
	
	/**The probability an agent will relocate*/
	public double relocation;
	
	/**The benefit multiplier used for achieving benefits (e.g., economic or cultural) by being located in a given settlement*/
	public double benefitMultiplier;
	
	/**Map used to keep track of flow to a settlement*/
	public Map<Settlement,Double>flowToSettlementProbability
		=new HashMap<Settlement,Double>();
	
	/**
	 * Constructor used to instantiate an agent
	 * @param id the agent id
	 * @param SettlementId the settlement id the agent is located in initially
	 */
	public Agent(int id, int SettlementId){
		this.agentNumber=id;
		this.originalSettlement=SettlementId;
	}
	
	public int getOriginalSettlement() {
		return originalSettlement;
	}
	public void setOriginalSettlement(int originalSettlement) {
		this.originalSettlement = originalSettlement;
	}
	public int getAgentNumber() {
		return agentNumber;
	}
	public void setAgentNumber(int agentNumber) {
		this.agentNumber = agentNumber;
	}
	
	
	public Settlement getSettlement() {
		return settlement;
	}

	public void setSettlement(Settlement Settlement) {
		this.settlement = Settlement;
	}
	
	

	public double getBenefitMultiplier() {
		return benefitMultiplier;
	}

	public void setBenefitMultiplier(double benefitMultiplier) {
		this.benefitMultiplier = benefitMultiplier;
	}

	public double getFlow() {
		return flow;
	}

	public void setFlow(double flow) {
		this.flow = flow;
	}

	
	public double getRelocation() {
		return relocation;
	}

	public void setRelocation(double relocation) {
		this.relocation = relocation;
	}

	public Map<Settlement, Double> getFlowToSettlementProbability() {
		return flowToSettlementProbability;
	}

	public void setFlowToSettlementProbability(Map<Settlement, Double> flowToSettlementProbability) {
		this.flowToSettlementProbability = flowToSettlementProbability;
	}

	@ScheduledMethod(start = 1, interval=1, pick=ScheduledMethod.ALL, priority = 3)
	public void step(){
		if(model==null)
			model=new AgentSettlementModel();
		
		model.run(this);
	}
	
	@ScheduledMethod(start = 1, interval=1, pick=ScheduledMethod.ALL, priority = 1)
	public void stepTwo(){
		
		model.runSecond(this);
	}
}

