package ucl.model;

import ucl.entities.Settlement;
import ucl.entities.Agent;

/**
 * Model to calculate a settlement's trade and social-ecological circumstances
 * via people movement and interaction.
 * 
 * @author 
 *
 */
public class SettlementModel implements Model{

	/**
	 * A type of cost (e.g., having to move items) in running interactions between settlements
	 */
	public static double operationCost=1.0;
	
	/**Total flow calculated for the entire simulation*/
	public static double totalFlow=0.0;
	
	/**
	 * Step 1. Clear settlement flow from previous runs.
	 */
	public void run(Object c){
		clearFlow((Settlement)c);
	}
	
	/**
	 * 
	 */
	public void runSecond(Object c){
	
	}
	
	/**
	 * This method calculates the social and environmental weights and benefits for a given
	 * place (c) for an agent (p). This looks at people in the city from your origin city
	 * and the benefits of their location in that city for a given agent. However, a city's size
	 * and exogenous benefits are also considered as environmental variables.
	 * 
	 * @param c the city c
	 */
	public void calculateSocioEnvironment(Settlement c,Agent p){
		double totalP=1;
		
		if(!c.getPeopleCities().containsKey(p.originalSettlement)){
			totalP=(double)c.getPeople().size()*c.getSettlementBenefit();
				
			c.rangeOfChoice.put(p, totalP);
			return;
		}

		else{
			double people=(double)c.getPeople().size()+(double)c.getPeopleCities().get(p.originalSettlement);
			totalP=people*p.getBenefitMultiplier()*c.getSettlementBenefit();
				
			c.rangeOfChoice.put(p, totalP);
		}
			
	}
	
	/**
	 * This calculates net flow for a city based on some operation cost for each person
	 * (operationCost) subtracted from their flow and then 
	 * aggregated for all people in the entire settlement to give the total settlement flow.
	 */
	public void calculateTotalNetFlow(Settlement c){
//		Iterator<Person> ic = c.people.iterator();
//		double flow=0;
//		while(ic.hasNext()){
//			Person p = ic.next();
//			p.flow=p.flow-operationCost;
//			flow+=p.flow;
//		}
//		c.flow=0;
	}
	
	public void clearFlow(Settlement c){
		c.flow=0;
		totalFlow=0.0;
	}
}
