package ucl.output;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;

import org.geotools.data.FeatureSource;
import org.geotools.data.shapefile.ShapefileDataStore;
import org.geotools.feature.*;
import org.geotools.feature.simple.SimpleFeatureImpl;
import org.opengis.feature.Feature;
import org.opengis.feature.simple.SimpleFeature;
import org.opengis.feature.type.AttributeType;
import org.opengis.feature.type.FeatureType;

/**
 * Shape File loader to parse shapefile input data
 * @author 
 *
 */
public class ShapeFileLoader{

	/**
	 * Method to load a shapefile using geotools functions.
	 * @param f the file you want to load
	 * @return a list with maps that have specific cell values from the shapefile
	 */
	public static List<Map<String,Object>> shpLoader(File f){
	List<Map<String,Object>> map = new ArrayList<Map<String,Object>>();
	Map<String, Object> data = new HashMap<String, Object>();
	URL shapeURL = null;
	
	try{
		shapeURL = f.toURI().toURL();
		
		// get feature results from the shapefile
		ShapefileDataStore store = new ShapefileDataStore(shapeURL);
		List<AttributeType>at=store.getSchema().getTypes();
		String name = store.getTypeNames()[0];
		FeatureSource source = store.getFeatureSource(name);
		FeatureCollection fsShape = source.getFeatures();
		FeatureIterator reader = fsShape.features();
	
		FeatureType tp;
		
		//read through the features and get the attributes
		while (reader.hasNext()) {
			SimpleFeatureImpl feature = (SimpleFeatureImpl) reader.next();
			tp= feature.getType();
			
			for(int i=0; i < at.size(); i++) {
	     		AttributeType atT=at.get(i);
				data.put(atT.getName().toString(),feature.getAttribute(atT.getName()));
			}
			data.put("the_geom", feature.getDefaultGeometry());
	        map.add(data);
			data = new HashMap<String, Object>();
	}

	//	System.out.println(map);

//	reader.close();
	}catch(MalformedURLException e){
		System.out.println("Cannot find shape file");
	}catch(IOException e){
		System.out.println("Features not found");
	}catch (NoSuchElementException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();

	} 

	//return the data in a map
	return map;
   }

	/**
	 * Alternative method for shapefile loading.
	 * @param f the file
	 * @param type the type of file
	 * @return a feature collection with the given data
	 */
	public static FeatureCollection shpLoader(File f, String type){
		FeatureCollection  fc = null;
		URL shapeURL = null;
		FeatureSource source;
		 try{
			 shapeURL = f.toURI().toURL();

			// get feature results
			ShapefileDataStore store = new ShapefileDataStore(shapeURL);
	//		List<AttributeType> at=store.getSchema().getTypes();
			String name = store.getTypeNames()[0];
    		source = store.getFeatureSource(name);
			fc = source.getFeatures();

		 }catch(MalformedURLException e){
				System.out.println("Cannot find shape file");
		}catch(IOException e){
				System.out.println("Features not found");
		}
		return fc;		
	}

	/**
	 * Method to parse a shapefile feature for its attributes.
	 * @param f the feature
	 * @param at the attribute type
	 * @return a map with the parsed data
	 */
	public static Map parseFeature(SimpleFeature f, AttributeType at[]){
		Map<String, Object> data = new HashMap<String, Object>();

		//this returns the attributes of a given feature
		for(int i=0; i < at.length; i++) {
			data.put(at[i].getName().toString(),f.getProperty(at[i].getName()));
		}
		
		//the attributes are returned in a map holding the data
		return data;
	}

}

